{--------------------------------------------------------------------------
 | Copyright (C) 1992,1993,1994,1995,1996,1997 Dallas Semiconductor Corporation.
 | All rights Reserved. Printed in U.S.A.
 | This software is protected by copyright laws of
 | the United States and of foreign countries.
 | This material may also be protected by patent laws of the United States
 | and of foreign countries.
 | This software is furnished under a license agreement and/or a
 | nondisclosure agreement and may only be used or copied in accordance
 | with the terms of those agreements.
 | The mere transfer of this software does not imply any licenses
 | of trade secrets, proprietary technology, copyrights, patents,
 | trademarks, maskwork rights, or any other form of intellectual
 | property whatsoever. Dallas Semiconductor retains all ownership rights.
 |--------------------------------------------------------------------------


}
unit Ibgenun;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, iBTMEXPW, AboutUn, GenGlob,
  Menus, RegUn, TabNotBk, Spin, ComCtrls, PortUn;

type
  { types for object connected to registration number }
  PRegNumInfo = ^RegNumInfo;  { pointer to reg number info }
  RegNumInfo = Record         { record of reg number info }
     ROM : array[0..8] of SmallInt; { array of rom number }
     Age : Byte;  { age of rom number in list }
  End;

  { record type for info on each type of device }
  InfoType = record
    MemType   : Byte;
    MemPages  : SmallInt;
    StatPages : Byte;
    Viewer : Array [0..2] of Byte;
    Name : String;
  End;

  { types for object connected to the page }
  PPageInfo = ^PageInfo;              { pointer to page info }
  PageInfo = Record                    { record of page info }
    Pg : SmallInt;                              { page number }
    NextPg : SmallInt;                    { next page pointer }
    ReDirect : SmallInt;          { redirected to page number }
    PgLen : SmallInt;                           { page length }
    PgType : SmallInt; { page type 0 scratch, 1 ram, 2 status }
    PgData : array[0..32] of Byte;      { array of page data }
    DataColor : array[0..32] of SmallInt;  { array of page data }
    DoStrong : Boolean;            { flag to do strongaccess }
    WriteCnt : SmallInt;                   { byte write count }
    Display: String;                          { label to show }
  End;

  TiBGenForm = class(TForm)
    MainMenu: TMainMenu;
    MenuFile: TMenuItem;
    MenuExit: TMenuItem;
    MenuHelp: TMenuItem;
    MenuAbout: TMenuItem;
    MenuOptions: TMenuItem;
    MenuPort: TMenuItem;
    CenterPanel: TPanel;
    TopPanel: TPanel;
    Image2: TImage;
    WLabel: TLabel;
    SaveDialog: TSaveDialog;
    OpenDialog: TOpenDialog;
    IdleTimer: TTimer;
    StatusBar: TStatusBar;
    NoDeviceLabel: TLabel;
    TabbedNotebook1: TTabbedNotebook;
    NoteBook: TPageControl;
    SetupPage: TTabSheet;
    SourcePage: TTabSheet;
    DestinationPage: TTabSheet;
    SourceGroup: TGroupBox;
    PageGroup: TGroupBox;
    StartLabel: TLabel;
    EndLabel: TLabel;
    StartSpin: TSpinEdit;
    EndSpin: TSpinEdit;
    PagesRadio: TRadioGroup;
    DestGroup: TGroupBox;
    WPGroup: TGroupBox;
    WPStartLabel: TLabel;
    WPEndLabel: TLabel;
    WPStartSpin: TSpinEdit;
    WPEndSpin: TSpinEdit;
    WPRadio: TRadioGroup;
    PageList: TListBox;
    ReadButton: TButton;
    ReadFileButton: TButton;
    SaveButton: TButton;
    WriteButton: TButton;
    DataMemo: TMemo;
    TypeDestLabel: TLabel;
    ROMDestLabel: TLabel;
    TypeLabel: TLabel;
    ROMLabel: TLabel;
    StatusLabel: TLabel;
    RequiredSNLabel: TEdit;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure SelectButtonClick(Sender: TObject);
    procedure MenuExitClick(Sender: TObject);
    procedure MenuAboutClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    Procedure GetSizePos;
    procedure MenuPortClick(Sender: TObject);
    procedure StartSpinChange(Sender: TObject);
    Procedure ParseTypes;
    procedure FormResize(Sender: TObject);
    procedure ResizePage;
    procedure PageListClick(Sender: TObject);
    procedure DisplayPageItem(Num : SmallInt);
    procedure PageFormatCheck(Num : SmallInt);
    procedure ReCalPackets;
    Function  ReadMemory: SmallInt;
    Function  ReadPage(PInfo : PPageInfo): SmallInt;
    procedure ReadButtonClick(Sender: TObject);
    Function  FindPart(SpecificDevice: String): Boolean;
    procedure OldNoteBookChange(Sender: TObject; NewTab: Integer;
      var AllowChange: Boolean);
    procedure WriteButtonClick(Sender: TObject);
    procedure EndSpinChange(Sender: TObject);
    Function  WritePage(Pg : SmallInt; Buf : PChar; PgLen : SmallInt): SmallInt;
    Function  Empty(Pg : SmallInt): SmallInt;
    Function  ProgramPage(ListPg : SmallInt): SmallInt;
    procedure PagesRadioClick(Sender: TObject);
    procedure WPRadioClick(Sender: TObject);
    procedure WPStartSpinChange(Sender: TObject);
    procedure WPEndSpinChange(Sender: TObject);
    procedure SaveButtonClick(Sender: TObject);
    procedure ReadFileButtonClick(Sender: TObject);
    Function  ConvertToBin(instr: String; Buf : PChar): SmallInt;
    Function  HexCharToByte(ch : Char): Byte;
    procedure IdleTimerTimer(Sender: TObject);
    Function  WriteApplicationReg(ApplBuf : PChar): SmallInt;
    Function  LockApplicationReg: SmallInt;
    procedure NoDevice(state : Boolean);
    Function  LoadiButtonFileMap(FileName: String): Boolean;
    procedure NoteBookChanging(Sender: TObject; var AllowChange: Boolean);
    procedure NoteBookChange(Sender: TObject);
  private
  public
    { Public declarations }
    ROM : array[0..8] of SmallInt; { array of rom number }
    LTop,LLeft,LWidth,LHeight : SmallInt;
    PageBytes: array[0..32] of TLabel;
    PageLabels: array[0..32] of TLabel;
    MaxType : SmallInt;
    TypeList : Array[0..128] of Infotype;
    DBMode : SmallInt;            { debug mode }
    DBSub : SmallInt;             { sub mode }
    DBPage : SmallInt;            { debug page  }
    DBType : SmallInt;            { page type }
    ErrorCnt : SmallInt;          { count of errors in a row }
    ErrorType : SmallInt;         { type of repeated errors }
    DBDisplay : SmallInt;         { page being displayed }
    MemPages,MemType: SmallInt;
    TempInfo : PPageInfo;          { pointer to temporary object }
    CurrentListPg : SmallInt;       { current page list number writing }
    CopyAll : Boolean;             { flag that matches the copy radio buttons }
    WPAll : Boolean;               { flag that matches the wp radio buttons }
    AutoMode : Boolean;            { true if file auto loaded/write/exit }
  end;

var
  iBGenForm: TiBGenForm;


implementation

{$R *.DFM}

{-----------------------------------------------------------------}
{ Executed once at start of application }
{ (like main) }
{}
procedure TiBGenForm.FormCreate(Sender: TObject);
Var
   RetStr  : Array[0..120] of Char;
   TType,TNum,TLeft,TTop,TWidth,THeight,TState : LongInt;
   i : SmallInt;
   rstr : String;
begin
   { set default values }
   PAppName := 'iButton Copy Machine';
   StrPCopy(AppName,'iButton Copy Machine');
   MainMode := MD_CHECKPORT;
   AutoMode := False;

   {Check for defaults of port type and port number }
   {First try the TMEX general defaults and then iBRec specifics}
   rstr := ReadTMEXRegStr('DefaultPortType','-1');
   Try
      TType := StrToInt(rstr);
   Except
      TType := -1;
   End;
   TType := ReadRegLong('PortType',TType);

   rstr := ReadTMEXRegStr('DefaultPortNum','-1');
   Try
      TNum := StrToInt(rstr);
   Except
      TNum := -1;
   End;
   TNum := ReadRegLong('PortNum',TNum);

   { get main size/position }
   TTop := ReadRegLong('Top',0);
   TLeft := ReadRegLong('Left',0);
   TWidth := ReadRegLong('Width',0);
   THeight := ReadRegLong('Height',0);
   If ((TTop = 0) Or (TLeft = 0) Or (TWidth = 0) Or (THeight = 0) Or
       ((TWidth + TLeft) > Screen.Width) Or ((THeight + TTop) > Screen.Height)) Then
      Position := poDefaultPosOnly { BUG!!! if use poDefault }
   Else
   Begin
      Top := TTop;
      Left := TLeft;
      Width := TWidth;
      Height := THeight;
   End;
   TState := ReadRegLong('WindowState',WNORMAL);
   Case TState of
      WNORMAL : WindowState := wsNormal;
      WMAX : WindowState := wsMaximized;
      else WindowState := wsNormal;
   End;

   { Decide how get port starts }
   If ((TType = -1) Or (TNum = -1)) Then
      PortMode := P_SHOW_GET_PORT
   Else
   Begin
      PortMode := P_GET_SESSION;
      PortNum := TNum;
      PortType := TType;
   End;

   { create the data labels }
   For i := 0 to 31 Do
   Begin
      PageBytes[i] := TLabel.Create(Application);
      PageBytes[i].Parent := TypeLabel.Parent;
      PageBytes[i].AutoSize := False;
      PageBytes[i].ParentFont := False;
      PageBytes[i].Font.Name := 'Arial';
      PageBytes[i].Font.Size := 1;
      PageBytes[i].Font.Style := [fsBold];
      PageBytes[i].Alignment := taCenter;
      PageBytes[i].Font.Color := clWhite;
      PageBytes[i].Top := 1;
      PageBytes[i].Left := 1;
      PageBytes[i].Visible := True;
      PageBytes[i].Enabled := True;
      PageBytes[i].Transparent := False;
      PageBytes[i].WordWrap := False;

      PageLabels[i] := TLabel.Create(Application);
      PageLabels[i].Parent := TypeLabel.Parent;
      PageLabels[i].AutoSize := False;
      PageLabels[i].ParentFont := False;
      PageLabels[i].Font.Name := 'Arial';
      PageLabels[i].Font.Size := 1;
      PageLabels[i].Font.Color := clBlack;
      PageLabels[i].Font.Style := [];
      PageLabels[i].Alignment := taCenter;
      PageLabels[i].Color := clBtnFace;
      PageLabels[i].Top := 1;
      PageLabels[i].Caption := IntToStr(i) + Chr($0D);
      PageLabels[i].Left := 1;
      PageLabels[i].Visible := True;
      PageLabels[i].Enabled := True;
      PageLabels[i].Transparent := False;
      PageLabels[i].WordWrap := False;
   End;

   { parse the device types }
   ParseTypes;

   { set default mode }
   DBMode := DB_CHECKPORT;
   DBSub := DBS_CREATE;
   DBType := NONE;
   NoteBook.ActivePage := SetupPage;
   CopyAll := True;
   WPAll := True;

   { Check command line parameter for a filename }
   If (ParamCount > 0) Then
   Begin
      If (FileExists(ParamStr(1))) Then
      Begin
         { attempt to use this as a iButton File }
         If (LoadiButtonFileMap(ParamStr(1))) Then
         Begin
            { check for an AUTO flag }
            If (ParamCount > 1) Then
            Begin
               If (CompareText(ParamStr(2),'AUTO') = 0) Then
               Begin
                  { yes, auto mode so minimize the app }
                  AutoMode := True;
                  { minimize, and start the write }
                  WindowState := wsMinimized;
               End;
            End;
         End
         Else
         Begin
            ShowMessage('ERROR, data not valid in file (' + ParamStr(1) + ') provided on the command line!');
            Application.terminate;
         End;
      End
      Else
      Begin
         ShowMessage('ERROR, file (' + ParamStr(1) + ') provided on the command line does not exist!');
         Application.terminate;
      End;
   End;

   { enable the timer }
   IdleTimer.Enabled := True;
end;

{-----------------------------------------------------------------}
{ temporary procedure, does nothing }
{}
procedure TiBGenForm.SelectButtonClick(Sender: TObject);
begin
   ShowMessage('Feature not supported yet.');
end;

{-----------------------------------------------------------------}
{ Exit
}
procedure TiBGenForm.MenuExitClick(Sender: TObject);
begin
   Application.Terminate;
end;

{----------------------------------------------------------------------------}
{ Show the about box
}
procedure TiBGenForm.MenuAboutClick(Sender: TObject);
begin
   AboutBox.Show;
end;

{----------------------------------------------------------------------------}
{ Save the status info
}
procedure TiBGenForm.FormDestroy(Sender: TObject);
Var
   tstr : Array[0..120] of Char;
   TState : LongInt;
begin
   GetSizePos; { Keep track of size/position }

   { save the PortType and PortNum }
   WriteRegLong('PortType',PortType);
   WriteRegLong('PortNum',PortNum);
   { size and position of control window }
   if ((LTop <> 0) And (LLeft <> 0) And
       (LWidth <> 0) And (LHeight <> 0) And
       (WindowState = wsNormal)) Then
   Begin
      WriteRegLong('Top',Top);
      WriteRegLong('Left',Left);
      WriteRegLong('Width',Width);
      WriteRegLong('Height',Height);
   End;
   Case WindowState of
      wsNormal : TState := WNORMAL;
      wsMaximized: TState := WMAX;
      else TState := WNORMAL;
   End;
   WriteRegLong('WindowState',TState);
end;

{----------------------------------------------------------------------------}
{ Get the size and position of this device viewer if normal
}
Procedure TiBGenForm.GetSizePos;
Begin
   { Keep track of size/position }
   If (WindowState = wsNormal) Then
   Begin
      LTop := Top;
      LLeft := Left;
      LWidth := Width;
      LHeight := Height;

      If ((Width = 0) Or (Height = 0)) Then
      Begin
         LWidth := -1;
         LHeight := -1;
      End;
   End;
End;

{----------------------------------------------------------------------------}
{ Choose a new port
}
procedure TiBGenForm.MenuPortClick(Sender: TObject);
begin
   PortForm.TPortNum := PortNum;
   PortForm.TPortType := PortType;

   DBMode := DB_CHECKPORT;
   PortMode := P_SHOW_GET_PORT;
end;


{-----------------------------------------------------------------}
{ Parse the type list
}
Procedure TiBGenForm.ParseTypes;
Var
   i,fam,t : SmallInt;
Begin
   { zero the list }
   For i := 0 to 128 do
   Begin
      TypeList[i].MemType := 0;
      TypeList[i].MemPages := 0;
      TypeList[i].StatPages := 0;
      TypeList[i].Viewer[0] := VIEW_NONE;
      TypeList[i].Viewer[1] := VIEW_NONE;
      TypeList[i].Viewer[2] := VIEW_NONE;
      TypeList[i].Name := '';
   End;

   { calculate the number of types in the config file }
   MaxType := DataMemo.Lines.Count div 14;

   For t := 0 to MaxType-1 Do
   Begin
      { get the family type number}
      fam := ((HexCharToByte(DataMemo.Lines[t * 14 + 1][1]) shl 4) Or
             (HexCharToByte(DataMemo.Lines[t * 14 + 1][2])));
      If (fam < 0) Or (fam > 128) Then
         Continue;

      { use the list name }
      TypeList[fam].Name := DataMemo.Lines[t * 14];

      { get memtype }
      Try
         TypeList[fam].MemType := StrToInt(DataMemo.Lines[t * 14 + 2]);
      Except
         TypeList[fam].MemType := 0;
      End;

      { get number of pages }
      Try
         TypeList[fam].MemPages := StrToInt(DataMemo.Lines[t * 14 + 3]);
      Except
         TypeList[fam].MemPages := 0;
      End;

      { get number of STATUS pages }
      Try
         TypeList[fam].StatPages := StrToInt(DataMemo.Lines[t * 14 + 4]);
      Except
         TypeList[fam].StatPages := 0;
      End;

      { get the viewer type }
      For i := 0 to 2 do
      Begin
         If (CompareStr('TEMPERATURE',DataMemo.Lines[t * 14 + 5 + i]) = 0) Then
            TypeList[fam].Viewer[i] := VIEW_TEMP
         Else If (CompareStr('FILE',DataMemo.Lines[t * 14 + 5 + i]) = 0) Then
            TypeList[fam].Viewer[i] := VIEW_FILE
         Else If (CompareStr('DEBUG',DataMemo.Lines[t * 14 + 5 + i]) = 0) Then
            TypeList[fam].Viewer[i] := VIEW_DEBUG
         Else If (CompareStr('SWITCH',DataMemo.Lines[t * 14 + 5 + i]) = 0) Then
            TypeList[fam].Viewer[i] := VIEW_SWITCH
         Else If (CompareStr('DS1991',DataMemo.Lines[t * 14 + 5 + i]) = 0) Then
            TypeList[fam].Viewer[i] := VIEW_DS1991
         Else If (CompareStr('TIME',DataMemo.Lines[t * 14 + 5 + i]) = 0) Then
            TypeList[fam].Viewer[i] := VIEW_TIME
         Else If (CompareStr('UDP',DataMemo.Lines[t * 14 + 5 + i]) = 0) Then
            TypeList[fam].Viewer[i] := VIEW_UDP
         Else
            TypeList[fam].Viewer[i] := VIEW_NONE;
      End;
   End;
End;

{----------------------------------------------------------------------------}
{ Resize the PageByte labels
}
procedure TiBGenForm.ResizePage;
Var
   w,i,t,o : SmallInt;
begin
   If (Visible) Then
   Begin
      WLabel.Font.Size := 1;
      w := NoteBook.ClientWidth div 33;
      While ((WLabel.Width < w) And (WLabel.Font.Size < 72)) Do
         WLabel.Font.Size := WLabel.Font.Size + 1;
      WLabel.Font.Size := WLabel.Font.Size - 1;
      t := (100 - WLabel.Height * 3) div 2;
      o := w div 2;

      For i := 0 to 31 Do
      Begin
         PageLabels[i].Font.Size := WLabel.Font.Size - 1;
         PageLabels[i].Height := WLabel.Height;
         PageLabels[i].Width := w;
         PageLabels[i].Top := t;
         PageLabels[i].Left := i * w + o;
         PageBytes[i].Font.Size := WLabel.Font.Size;
         PageBytes[i].Width := w;
         PageBytes[i].Height := WLabel.Height * 2;
         PageBytes[i].Top := t + WLabel.Height;
         PageBytes[i].Left := i * w + o;
      End;
   End;
End;

{----------------------------------------------------------------------------}
{ Resize
}
procedure TiBGenForm.FormResize(Sender: TObject);
begin
   ResizePage;
end;

{----------------------------------------------------------------------------}
{ Display the selected indicated page
}
procedure TiBGenForm.DisplayPageItem(Num : SmallInt);
Var
   i,j : SmallInt;
   PInfo : PPageInfo;
Begin
   If (PageList.Items.Count >= Num) And (Num >= 0) Then
   Begin
      { get a pointer to the page object }
      PInfo := PPageInfo(PageList.Items.Objects[Num]);

      { loop for each character }
      For i := 0 to (PInfo^.PgLen - 1) Do
      Begin
         { set text }
         If ((PInfo^.PgData[i] <= Byte($20)) Or (PInfo^.PgData[i] >= Byte($7F))) Then
            PageBytes[i].Caption :=  Chr($0D) + Format('%.2X',[PInfo^.PgData[i]])
         Else
            PageBytes[i].Caption :=  Chr(PInfo^.PgData[i]) + Chr($0D) + Format('%.2X',[PInfo^.PgData[i]]);

         { set color }
         If (PInfo^.DataColor[i] = CLR_LENGTH) Then
         Begin
            PageBytes[i].Color := clMaroon;
            PageBytes[i].Font.Color := clWhite;
         End
         Else If (PInfo^.DataColor[i] = CLR_NONPRINT) Then
         Begin
            PageBytes[i].Color := clPurple;
            PageBytes[i].Font.Color := clSilver;
         End
         Else If (PInfo^.DataColor[i] = CLR_PRINT) Then
         Begin
            PageBytes[i].Color := clNavy;
            PageBytes[i].Font.Color := clYellow;
         End
         Else If (PInfo^.DataColor[i] = CLR_CRC) Then
         Begin
            PageBytes[i].Color := clGreen;
            PageBytes[i].Font.Color := clWhite;
         End
         Else If (PInfo^.DataColor[i] = CLR_EMPTY) Then
         Begin
            PageBytes[i].Color := clAqua;
            PageBytes[i].Font.Color := clBlack;
         End;
         PageBytes[i].Visible := True;
         PageLabels[i].Visible := True;
      End;
      { remove the rest of the labels }
      For j := (PInfo^.PgLen - 1) to 31 Do
      Begin
         PageBytes[j].Visible := False;
         PageLabels[j].Visible := False;
      End;

      DBDisplay := Num;
   End;
End;

{----------------------------------------------------------------------------}
{ Check the page for colorization for multi-page packets
}
procedure TiBGenForm.ReCalPackets;
Var
   PInfo : PPageInfo;
   startpg,curpg,i,j,k,len,cnt,pgcnt,listnum,startlist : SmallInt;
Begin
   { loop through all of the pages }
   For i := 0 to PageList.Items.Count - 1 Do
   Begin
      { get a pointer to the page object }
      PInfo := PPageInfo(PageList.Items.Objects[i]);

      { only look at non-used regular MEMORY pages }
      If ((PInfo^.PgType = MEMORY) And
          (PInfo^.DataColor[0] = CLR_EMPTY)) Then
      Begin
         { start the page counters }
         startpg := PInfo^.Pg;
         curpg := PInfo^.Pg;
         CRC16 := PInfo^.Pg;
         listnum := i;
         startlist := i;

         { assume the first byte is length }
         PInfo^.DataColor[0] := CLR_LENGTH;
         len := PInfo^.PgData[0];
         pgcnt := 1;
         DoCRC16(PInfo^.PgData[0]);

         { check for extended length format }
         If (len = 255) Then
         Begin
            pgcnt := 2;
            PInfo^.DataColor[1] := CLR_LENGTH;
            len := len + PInfo^.PgData[1];
            DoCRC16(PInfo^.PgData[1]);
         End;
         cnt := 0;

         { check for invalid packet length }
         If (len <= 508) Then
         Begin
            { loop to read all of the data and CRC bytes }
            Repeat
               { if not within 2 of the end call it data }
               If (cnt < len) Then
               Begin
                  If ((PInfo^.PgData[pgcnt] < Byte($20)) Or (PInfo^.PgData[pgcnt] >= Byte($7F))) Then
                     PInfo^.DataColor[pgcnt] :=  CLR_NONPRINT
                  Else
                     PInfo^.DataColor[pgcnt] :=  CLR_PRINT;
               End
               Else
                  PInfo^.DataColor[pgcnt] := CLR_CRC;

               { perform the crc16 }
               DoCRC16(PInfo^.PgData[pgcnt]);

               { increment the counters }
               Inc(pgcnt);
               Inc(cnt);

               { check to see if need another page }
               If (pgcnt >= 32) And (cnt <> (len + 2)) Then
               Begin
                  { check for end of regular memory }
                  If ((curpg + 1) >= MemPages) Then
                     Break
                  Else
                  Begin
                     Inc(curpg);
                     Inc(listnum);
                     pgcnt := 0;
                     { check for going past end of list }
                     If (listnum >= PageList.Items.Count) Then
                     Begin
                        Dec(listnum);
                        Break;
                     End;
                     { get a pointer to the page object }
                     PInfo := PPageInfo(PageList.Items.Objects[listnum]);
                     { make sure not going to step on good packets }
                     If (PInfo^.DataColor[0] <> CLR_EMPTY) Then
                        Break;
                  End;
               End;
            Until (cnt = (len + 2));
         End;

         { on success mark the string on the pages }
         If (CRC16 = $B001) Then
         Begin
            If (startlist <> listnum) Then
            Begin
               For j := startlist To listnum Do
               Begin
                  { get a pointer to the page object }
                  PInfo := PPageInfo(PageList.Items.Objects[j]);
                  { set the string to display in the list }
                  PInfo^.Display :=  'Page ' + Format('%2d',[PInfo^.Pg]) +
                      '    Mult-page CRC packet (' +
                      Format('%2d of %2d)',[(j - startlist+1),(listnum - startlist+1)]);
                  PageList.Items.Strings[j] := PInfo^.Display;
               End;
            End;
            { check for over end of for loop }
            If (i >= (PageList.Items.Count - 1)) Then
               Break;
         End
         { check for a failure, clear all of the colors for those pages }
         Else
         Begin
            For j := startlist To listnum Do
            Begin
               { get a pointer to the page object }
               PInfo := PPageInfo(PageList.Items.Objects[j]);
               { loop to clear every byte }
               For k := 0 to PInfo^.PgLen - 1 Do
                  PInfo^.DataColor[k] := CLR_EMPTY;
            End;
         End;
      End;
   End;
End;

{----------------------------------------------------------------------------}
{ Check the page for colorization
}
procedure TiBGenForm.PageFormatCheck(Num : SmallInt);
Var
   i : SmallInt;
   PInfo : PPageInfo;
Begin
   If (PageList.Items.Count >= Num) Then
   Begin
      { get a pointer to the page object }
      PInfo := PPageInfo(PageList.Items.Objects[Num]);

      { make all empty to start }
      For i := 0 to (PInfo^.PgLen - 1) Do
         PInfo^.DataColor[i] := CLR_EMPTY;

      If (PInfo^.PgData[0] <= 29) Then
      Begin
         { attempt to see if valid packet }
         CRC16 := PInfo^.Pg;
         For i := 0 to (PInfo^.PgData[0] + 2) Do
            DoCRC16(PInfo^.PgData[i]);
         If (CRC16 = $B001) Then
         Begin
            PInfo^.DataColor[0] := CLR_LENGTH;
            For i := 1 to PInfo^.PgData[0] Do
            Begin
               If ((PInfo^.PgData[i] < Byte($20)) Or (PInfo^.PgData[i] >= Byte($7F))) Then
                  PInfo^.DataColor[i] :=  CLR_NONPRINT
               Else
                  PInfo^.DataColor[i] :=  CLR_PRINT;
            End;
            PInfo^.DataColor[PInfo^.PgData[0]+1] := CLR_CRC;
            PInfo^.DataColor[PInfo^.PgData[0]+2] := CLR_CRC;
         End;
      End;

      { try for a Lock type packet }
      If (PInfo^.PgLen = 32) And (PInfo^.DataColor[0] = CLR_EMPTY) Then
      Begin
         CRC16 := PInfo^.Pg;
         For i := 0 to 31 Do
            DoCRC16(PInfo^.PgData[i]);
         If (CRC16 = $B001) Then
         Begin
            For i := 0 to 29 Do
            Begin
               If ((PInfo^.PgData[i] < Byte($20)) Or (PInfo^.PgData[i] >= Byte($7F))) Then
                  PInfo^.DataColor[i] :=  CLR_NONPRINT
               Else
                  PInfo^.DataColor[i] :=  CLR_PRINT;
            End;
            PInfo^.DataColor[30] := CLR_CRC;
            PInfo^.DataColor[31] := CLR_CRC;
            { adjust the display to indicate a Chris packet }
            PInfo^.Display :=  'Page ' + Format('%2d',[PInfo^.Pg]) +
                   '    Lock-Test Mode Packet (no length byte)';
            PageList.Items.Strings[num] := PInfo^.Display;
         End;
      End;
   End;
End;

{----------------------------------------------------------------------------}
{ Read the memory contents
}
Function TiBGenForm.ReadMemory: SmallInt;
Var
   tb : Byte;
   i,rt,flag : SmallInt;
   tstr : String;
Begin
   rt := DBIN_PROGRESS;

   { create the object for the page }
   If (DBSub = DBS_CREATE) Then
   Begin
      TempInfo := New(PPageInfo);

      { check if first page created }
      If (DBType = NONE) Then
      Begin
         If (MemType = MEM_EE) Then
         Begin
            DBType := APPL;
            TempInfo^.PgLen := 8;
         End
         Else
         Begin
            DBType := MEMORY;
            TempInfo^.PgLen := 32;

            If (Not CopyAll) And
               (StartSpin.Value < MemPages) Then
               DBPage := StartSpin.Value
            Else
               DBPage := 0;
         End;
      End
      { is memory or scratch }
      Else If (DBType = MEMORY) Then
      Begin
         TempInfo^.PgLen := 32;
      End
      { must be status }
      Else If (DBType = STATUS) Then
         TempInfo^.PgLen := 8;


      TempInfo^.PgType := DBType;
      TempInfo^.Pg := DBPage;
      TempInfo^.ReDirect := -1;
      TempInfo^.DoStrong := True;

      DBSub := DBS_READ;
   End
   { read the page }
   Else If (DBSub = DBS_READ) Then
   Begin
      { read the current page }
      flag := ReadPage(TempInfo);

      { check if done with this page }
      If (flag = DBSUCCESS) Then
         DBSub := DBS_DISPLAY
      Else
         rt := flag;
   End
   { add the page to the list }
   Else If (DBSub = DBS_DISPLAY) Then
   Begin
      { add to the listbox }
      flag := PageList.Items.AddObject(TempInfo^.Display, Pointer(TempInfo));

      { Check this page for format }
      PageFormatCheck(flag);

      { check if this is the first page to be displayed }
      If (DBDisplay = -1) Then
         DisplayPageItem(0);

      { is scratch so memory next }
      If ((TempInfo^.PgType = SCRATCH) Or
          (TempInfo^.PgType = APPL)) Then
      Begin
         DBPage := 0;
         DBSub := DBS_CREATE;
         DBType := MEMORY;
      End
      { memory, check if end or status }
      Else If (TempInfo^.PgType = MEMORY) Then
      Begin
         { not at end }
         If (TempInfo^.Pg < (MemPages - 1)) Then
         Begin
            { depending on setup, do we read another page }
            If (CopyAll) Or
               (TempInfo^.Pg < EndSpin.Value)  Then
            Begin
               DBPage := TempInfo^.Pg + 1;
               DBSub := DBS_CREATE;
            End
            Else
               rt := DBDONE;
         End
         { at end }
         Else
            rt := DBDONE;
      End
      { at end }
      Else
         rt := DBDONE;
   End;

   ReadMemory := rt;
End;

{----------------------------------------------------------------------------}
{ Read the current page
}
Function TiBGenForm.ReadPage(PInfo : PPageInfo): SmallInt;
Var
   tb : Byte;
   i,rt,flag : SmallInt;
   Buf : array[0..50] of Byte;      { array of page data }
   tstr : String;
Begin
   rt := DBSUCCESS;

   {attempt to get a session }
   SHandle := TMExtendedStartSession(PortNum,PortType,nil);
   If (SHandle > 0) Then
   Begin
      { set the rom }
      TMRom(SHandle,@StateBuf,@ROM);

      { verify and select the device }
      If (PInfo^.DoStrong) Then
         flag := TMStrongAccess(SHandle,@StateBuf)
      Else
         flag := TMAccess(SHandle,@StateBuf);
      { toggle the do strong flag }
      PInfo^.DoStrong := Not PInfo^.DoStrong;

      If (flag = 1) Then
      Begin
         { check what type of memory device }
         If ((MemType = MEM_NV) Or (MemType = MEM_EE) Or
             ((MemType = MEM_MNV) And (PInfo^.PgType = SCRATCH))) Then
         Begin
            { scratch }
            If (PInfo^.PgType = SCRATCH) Then
            Begin
               PInfo^.Display := 'Scatchpad ';
               PInfo^.NextPg := 0;
               TMTouchByte(SHandle,$AA);
               PInfo^.Pg := TMTouchByte(SHandle,$FF);
               If (MemType <> MEM_EE) Then
                  PInfo^.Pg := PInfo^.Pg Or (TMTouchByte(SHandle,$FF) shl 8);
               TMTouchByte(SHandle,$FF);
            End
            { regular memory }
            Else If (PInfo^.PgType = MEMORY) Then
            Begin
               PInfo^.Display :=  'Page ' + Format('%2d',[PInfo^.Pg]);
               TMTouchByte(SHandle,$F0);
               TMTouchByte(SHandle,(PInfo^.Pg shl 5) And $FF);
               If (MemType <> MEM_EE) Then
                  TMTouchByte(SHandle,(PInfo^.Pg shr 3) And $FF);
               { set next page }
               If (PInfo^.Pg < MemPages) Then
                  PInfo^.NextPg := PInfo^.Pg + 1
               Else
                  PInfo^.NextPg := 0;
            End
            { application register }
            Else If (PInfo^.PgType = APPL) Then
            Begin
               PInfo^.Display := 'Application Register ';
               PInfo^.NextPg := 0;
               TMTouchByte(SHandle,$C3);
               PInfo^.Pg := TMTouchByte(SHandle,$00);
            End;

            { loop to read }
            For i := 0 to PInfo^.PgLen - 1 Do
            Begin
               tb := Byte(TMTouchByte(SHandle,$FF));
               If (tb <> PInfo^.PgData[i]) Then
               Begin
                  rt := DBIN_PROGRESS;
                  PInfo^.PgData[i] := tb;
               End;
            End;
         End
         { self-generated CRC devices }
         Else
         Begin
            { regular memory }
            If (PInfo^.PgType = MEMORY) Then
            Begin
               flag := TMExtendedReadPage(SHandle,@StateBuf,PInfo^.Pg,@Buf[0],0);
               If (flag > 0) Then
               Begin
                  { Transaction TM exception }
                  If (MemType = MEM_MNV) Then
                  Begin
                     PInfo^.PgLen := 32;
                     If (PInfo^.Pg < MemPages) Then
                        PInfo^.NextPg := PInfo^.Pg + 1
                     Else
                        PInfo^.NextPg := 0;
                     tstr := 'Page ' + Format('%2d',[PInfo^.Pg]) + '   Counter:';
                     For i := 32 to 35 Do
                        tstr := tstr + Format('%.2X',[Buf[i]]);
                     tstr := tstr + '   Tamper:';
                     For i := 36 to 39 Do
                        tstr := tstr + Format('%.2X',[Buf[i]]);
                     PInfo^.Display := tstr;
                  End
                  Else
                  Begin
                     PInfo^.Display :=  'Page ' + Format('%2d',[PInfo^.Pg]);
                     PInfo^.PgLen := flag;
                     { set the next page and redirection }
                     If (Buf[32] <> 0) Then
                     Begin
                        PInfo^.ReDirect := Buf[32];
                        PInfo^.NextPg := Buf[32];
                        PInfo^.Display := PInfo^.Display + '-> Redirected to Page ' + IntToStr(PInfo^.ReDirect);
                     End
                     Else If (PInfo^.Pg < MemPages) Then
                        PInfo^.NextPg := PInfo^.Pg + 1
                     Else
                        PInfo^.NextPg := 0;
                  End;
                  For i := 0 to PInfo^.PgLen Do
                     PInfo^.PgData[i] := Buf[i];
                  rt := DBSUCCESS;
               End
               Else
                  rt := flag;
            End;
         End;
      End
      { no device }
      Else
      Begin
         PInfo^.DoStrong := True;
         rt := DBNO_DEVICE
      End;

      { close the opened session }
      TMEndSession(SHandle);
   End
   Else
      rt := DBNO_SESSION;

   ReadPage := rt;
End;

{----------------------------------------------------------------------------}
{ Read an EPROM page an verify that it is empty
}
Function TiBGenForm.Empty(Pg : SmallInt): SmallInt;
Var
   i,rt,flag : SmallInt;
   tstr : String;
   Buf : Array[0..35] of Byte;
Begin
   rt := DBSUCCESS;

   {attempt to get a session }
   SHandle := TMExtendedStartSession(PortNum,PortType,nil);
   If (SHandle > 0) Then
   Begin
      { read the page }
      flag := TMExtendedReadPage(SHandle,@StateBuf,Pg,@Buf[0],0);

      { look at all of the bytes for unprogrammed }
      For i := 0 to 31 Do
         If (Buf[i] <> $FF) Then
         Begin
            rt := DBNON_EMPTY;
            Break;
         End;

      { make sure not already redirected }
      If (Buf[32] <> $00) Then
         rt := DB_REDIRECT;

      { close the opened session }
      TMEndSession(SHandle);
   End
   Else
      rt := DBNO_SESSION;

   Empty := rt;
End;

{----------------------------------------------------------------------------}
{ Write an NVRAM page
}
Function TiBGenForm.WritePage(Pg : SmallInt; Buf : PChar; PgLen : SmallInt): SmallInt;
Var
   tb : Byte;
   i,rt,flag : SmallInt;
   tstr : String;
Begin
   rt := DBSUCCESS;

   {attempt to get a session }
   SHandle := TMExtendedStartSession(PortNum,PortType,nil);
   If (SHandle > 0) Then
   Begin
      { write the scratchpad at address Pg * PgLen }
      flag := WriteScratch(Pg * PgLen, Buf, PgLen, MemType);

      { copy the scratch if write was a success }
      if (flag > 0) Then
         flag := CopyScratch(Pg * PgLen, PgLen, MemType);

      If (flag < 0) Then
         rt := flag;

      { close the opened session }
      TMEndSession(SHandle);
   End
   Else
      rt := DBNO_SESSION;

   WritePage := rt;
End;

{----------------------------------------------------------------------------}
{ Write an EPROM page
}
Function TiBGenForm.ProgramPage(ListPg : SmallInt): SmallInt;
Var
   tb : Byte;
   i,rt,flag,bits,addr,wrbyte : SmallInt;
   PInfo : PPageInfo;
Begin
   rt := DBSUCCESS;
   bits := 8;

   { get a pointer to the page object }
   PInfo := PPageInfo(PageList.Items.Objects[ListPg]);

   StatusLabel.Caption := 'Writing EPROM page ' + IntToStr(PInfo^.Pg);

   {attempt to get a session }
   SHandle := TMExtendedStartSession(PortNum,PortType,nil);
   If (SHandle > 0) Then
   Begin
      { create the bitmap byte to write }
      If (MemPages <= 4) Then
      Begin
         addr := 0;
         wrbyte := Not ($01 shl (PInfo^.Pg + 4));
      End
      Else
      Begin
         addr := (PInfo^.Pg div 8) + 64;
         wrbyte := Not ($01 shl (PInfo^.Pg mod 8));
      End;

      { first set the Bitmap to used }
      flag := TMProgramByte(SHandle,@StateBuf,wrbyte,
                     addr,1,@bits,1);

      { loop to write each byte in the page starting where left off }
      For i := PInfo^.WriteCnt to PInfo^.PgLen - 1 Do
      Begin
         if (flag <> 1) Then
         Begin
            rt := flag;
            Break;
         End;

         flag := TMProgramByte(SHandle,@StateBuf,PInfo^.PgData[i],
                             PInfo^.Pg * PInfo^.PgLen + i,0,@bits,0);
         Inc(PInfo^.WriteCnt);
      End;

      { check if done with page, check on write protect }
      If (rt = DBSUCCESS) And
         (WPAll Or ((PInfo^.Pg >= WPStartSpin.Value) And
                    (PInfo^.Pg <= WPEndSpin.Value)       )) Then
      Begin
         { create the bitmap byte to write }
         If (MemPages <= 4) Then
         Begin
            addr := 0;
            wrbyte := Not ($01 shl PInfo^.Pg);
         End
         Else
         Begin
            addr := (PInfo^.Pg div 8);
            wrbyte := Not ($01 shl (PInfo^.Pg mod 8));
         End;

         { first set the writeprotect it }
         flag := TMProgramByte(SHandle,@StateBuf,wrbyte,
                               addr,1,@bits,1);
         if (flag <> 1) Then
            rt := flag;
      End;

      { close the opened session }
      TMEndSession(SHandle);
   End
   Else
      rt := DBNO_SESSION;

   ProgramPage := rt;
End;


{----------------------------------------------------------------------------}
{ Display the page that was clicked on
}
procedure TiBGenForm.PageListClick(Sender: TObject);
begin
   DisplayPageItem(PageList.ItemIndex);
end;


{----------------------------------------------------------------------------}
{ Find a part of a correct type
}
Function TiBGenForm.FindPart(SpecificDevice: String): Boolean;
Var
   TempROM : String;
   rt : Boolean;
   m,i : SmallInt;
Begin
   { start out with a negative return }
   rt := False;

   {attempt to get a session }
   SHandle := TMExtendedStartSession(PortNum,PortType,nil);
   If (SHandle > 0) Then
   Begin
      { find the next part }
      If (TMNext(SHandle,@StateBuf) > 0) Then
      Begin
         { read the rom }
         ROM[0] := 0;
         TMRom(SHandle,@StateBuf,@ROM);

         { check for the DS1410D brick ID }
         If ((ROM[0] = $81) And (ROM[6] = $00) And ((ROM[5] And $F0) = $00)) Then
            rt := False
         Else
            rt := True;

         { check if looking for a specific device }
         if (rt And (Length(Trim(SpecificDevice)) > 0)) Then
         Begin
            If (Length(Trim(SpecificDevice)) = 16) Then
            Begin
               { create the string to compare }
               TempROM := '';
               For i := 7 downto 0 do
                  TempROM := TempROM + IntToHex(ROM[i],2);

               If (CompareText(Trim(SpecificDevice),TempROM) <> 0) Then
                  rt := False;
            End
            Else
            Begin
               { disable timer while sevicing }
               IdleTimer.Enabled := False;
               ShowMessage('ERROR, the specified ROM number ' + SpecificDevice +
                           ' is not the correct length');
               DBMode := DB_IDLE;
               rt := False;
               { enable the timer }
               IdleTimer.Enabled := True;
            End;
         End;

      End;

      { close the opened session }
      TMEndSession(SHandle);
   End;

   { set return value }
   FindPart := rt;
End;


{----------------------------------------------------------------------------}
{ Find and Read the iButton
}
procedure TiBGenForm.ReadButtonClick(Sender: TObject);
Var
   PInfo : PPageInfo;
begin
   { check current state }
   If (ReadButton.Caption = 'Cancel Read') Then
   Begin
      ReadButton.Caption := 'Read Source &iButton';
      DBMode := DB_IDLE;
   End
   Else
   Begin
      ReadButton.Caption := 'Cancel Read';
      DBMode := DB_FINDSOURCE;
   End;

   { remove all of the pages in the pagelist }
   While (PageList.Items.Count > 0) do
   Begin
      { get a pointer to the page object }
      PInfo := PPageInfo(PageList.Items.Objects[0]);
      Dispose(PInfo);
      PageList.Items.Delete(0);
      DBSub := DBS_CREATE;
      DBType := NONE;
      ROMLabel.Caption := '';
      TypeLabel.Caption := '';
   End;
end;

{----------------------------------------------------------------------------}
{ Notebook tab change
}
procedure TiBGenForm.OldNoteBookChange(Sender: TObject; NewTab: Integer;
  var AllowChange: Boolean);
begin
end;

{----------------------------------------------------------------------------}
{ Find and write destination button
}
procedure TiBGenForm.WriteButtonClick(Sender: TObject);
begin
   { check to see if there is any data to write }
   If (PageList.Items.Count <= 0) Then
   Begin
      MessageBeep(MB_ICONEXCLAMATION);
      ShowMessage('ERROR, no data to write, go back to SOURCE tab!');
   End
   { got data }
   Else
   Begin
      { canceling? }
      If (WriteButton.Caption = 'Cancel Write') Then
      Begin
         WriteButton.Caption := 'Write Destination &iButton';
         StatusLabel.Caption := '';
         DBMode := DB_IDLE;
      End
      Else
      Begin
         WriteButton.Caption := 'Cancel Write';
         DBMode := DB_FINDDEST;
      End;

      ROMDestLabel.Caption := '';
      TypeDestLabel.Caption := '';
      StatusLabel.Caption := '';
   End;
end;


{-----------------------------------------------------------------}
{ Change the range state (all or selected)
}
procedure TiBGenForm.PagesRadioClick(Sender: TObject);
Var
   state : Boolean;
begin
   state := (PagesRadio.ItemIndex = 1);
   { set copyall flag and enable/disable spins }
   CopyAll := Not state;
   StartLabel.Enabled := state;
   EndLabel.Enabled := state;
   StartSpin.Enabled := state;
   EndSpin.Enabled := State;
end;

{-----------------------------------------------------------------}
{ Change the range state (all or selected)
}
procedure TiBGenForm.WPRadioClick(Sender: TObject);
Var
   state : Boolean;
begin
   state := (WPRadio.ItemIndex = 1);
   { set copyall flag and enable/disable spins }
   WPAll := Not state;
   WPStartLabel.Enabled := state;
   WPEndLabel.Enabled := state;
   WPStartSpin.Enabled := state;
   WPEndSpin.Enabled := State;
end;


{-----------------------------------------------------------------}
{ Change start page event, make sure it does not pass end page
}
procedure TiBGenForm.StartSpinChange(Sender: TObject);
begin
   If (StartSpin.Value > EndSpin.Value) Then
   Begin
      StartSpin.Value := EndSpin.Value;
      MessageBeep(MB_OK);
   End;

   { check on WP }
   If (WPStartSpin.Value < StartSpin.Value) Then
      WPStartSpinChange(Sender);
end;

{-----------------------------------------------------------------}
{ End page event, make sure it does not precede start
}
procedure TiBGenForm.EndSpinChange(Sender: TObject);
begin
   If (EndSpin.Value < StartSpin.Value) Then
   Begin
      EndSpin.Value := StartSpin.Value;
      MessageBeep(MB_OK);
   End;

   { check on WP }
   If (WPEndSpin.Value > EndSpin.Value) Then
      WPEndSpinChange(Sender);
end;

{-----------------------------------------------------------------}
{ Change start wp page event, make sure it does not pass end page
}
procedure TiBGenForm.WPStartSpinChange(Sender: TObject);
begin
   If (WPStartSpin.Value > WPEndSpin.Value) Then
   Begin
      WPStartSpin.Value := WPEndSpin.Value;
      MessageBeep(MB_OK);
   End;

   { check on WP }
   If (WPStartSpin.Value < StartSpin.Value) Then
      WPStartSpin.Value := StartSpin.Value;
end;

{-----------------------------------------------------------------}
{ Change end wp page event, make sure it does not pass end page
}
procedure TiBGenForm.WPEndSpinChange(Sender: TObject);
begin
   If (WPEndSpin.Value < WPStartSpin.Value) Then
   Begin
      WPEndSpin.Value := WPStartSpin.Value;
      MessageBeep(MB_OK);
   End;

   { check on WP }
   If (WPEndSpin.Value > EndSpin.Value) Then
      WPEndSpin.Value := EndSpin.Value;
end;

{-----------------------------------------------------------------}
{ Save button info to a file
}
procedure TiBGenForm.SaveButtonClick(Sender: TObject);
Var
   TempStrs : TStrings;
   PInfo : PPageInfo;
   i,j,k,lasttype,startloop : SmallInt;
   tstr : String;
begin
   { set the initial directory to the executables directory }
   SaveDialog.InitialDir := ExtractFilePath(Application.ExeName);

   { create the string list }
   TempStrs := TStringList.Create;
   Try
      { execute the dialog }
      If (SaveDialog.Execute) Then
      Begin
         { loop to write all of the data into a strings list }
         lasttype := -1;
         For i := 0 to (PageList.Items.Count - 1) Do
         Begin
            { get a pointer to the current object }
            PInfo := PPageInfo(PageList.Items.Objects[i]);
            If (PInfo^.PgType = APPL) Then
               tstr := Format('APPLICATION REGISTER=',[PInfo^.Pg])
            Else
               tstr := Format('Page %.3d=',[PInfo^.Pg]);
            { loop through the bytes in the page }
            For j := 0 to (PInfo^.PgLen - 1) Do
            Begin
               tstr := tstr + Format('%.2X ',[PInfo^.PgData[j]]);
            End;
            { add to the string list }
            TempStrs.Add(tstr);
         End;
         { save it to the selected file }
         TempStrs.SaveToFile(SaveDialog.FileName);
      End;
   Finally
      { free up the string list memory }
      TempStrs.Free;
   End;
end;

{-----------------------------------------------------------------}
{ Read button info to a file
}
procedure TiBGenForm.ReadFileButtonClick(Sender: TObject);
Var
   PInfo : PPageInfo;
   i,j,k,lasttype,pgcnt,len,newcount : SmallInt;
   tstr,pagestr : String;
   Buf : array[0..200] of Char;
begin
   { set the initial directory to the executables directory }
   SaveDialog.InitialDir := ExtractFilePath(Application.ExeName);

   { execute the dialog }
   If (OpenDialog.Execute) Then
   Begin
      LoadiButtonFileMap(OpenDialog.FileName);
   End;

   { select the first page }
   If (PageList.Items.Count > 0) Then
   Begin
      PageList.ItemIndex := 0;
      DisplayPageItem(PageList.ItemIndex);
   End;
   { check the format for coloring }
   ReCalPackets;
end;


{-----------------------------------------------------------------}
{ Read button info to a file
}
Function TiBGenForm.LoadiButtonFileMap(FileName: String): Boolean;
Var
   TempStrs : TStrings;
   PInfo : PPageInfo;
   i,j,k,lasttype,pgcnt,len,newcount : SmallInt;
   tstr,pagestr : String;
   Buf : array[0..200] of Char;
Begin
   { create the string list }
   TempStrs := TStringList.Create;
   Try
      { read the selected file }
      TempStrs.LoadFromFile(FileName);

      { remove all of the pages in the pagelist }
      While (PageList.Items.Count > 0) do
      Begin
         { get a pointer to the page object }
         PInfo := PPageInfo(PageList.Items.Objects[0]);
         Dispose(PInfo);
         PageList.Items.Delete(0);
         DBSub := DBS_CREATE;
         DBType := NONE;
         ROMLabel.Caption := '';
         TypeLabel.Caption := '';
      End;

      { loop to read each page of data until fail }
      lasttype := -1;
      pgcnt := -1;

      Repeat
         { construct a string for look for }
         { look for application register first }
         If (pgcnt = -1) Then
            tstr := 'APPLICATION REGISTER'
         Else
            tstr := Format('Page %.3d',[pgcnt]);
         { Search for page pgcnt }
         pagestr := TempStrs.Values[tstr];
         If (Length(pagestr) <> 0) Then
         Begin
            TempInfo := New(PPageInfo);
            { look for application register first }
            If (pgcnt = -1) Then
            Begin
               TempInfo^.PgLen := 8;
               TempInfo^.PgType := APPL;
               TempInfo^.Display :=  'Application Register ';
            End
            Else
            Begin
               TempInfo^.PgLen := 32;
               TempInfo^.PgType := MEMORY;
               TempInfo^.Display :=  'Page ' + Format('%2d',[pgcnt]);
            End;
            TempInfo^.ReDirect := -1;
            TempInfo^.DoStrong := True;
            TempInfo^.Pg := pgcnt;
            TempInfo^.DataColor[0] := CLR_EMPTY;

            { convert to a buf }
            len := ConvertToBin(pagestr,Buf);
            { check for good length }
            if (len <> TempInfo^.PgLen) Then
            Begin
               MessageBeep(MB_ICONEXCLAMATION);
               ShowMessage('ERROR, reading file, incorrect format.');
               Break;
            End;
            { loop to copy into object }
            For i := 0 to TempInfo^.PgLen - 1 Do
            Begin
               TempInfo^.PgData[i] := Byte(Buf[i]);
            End;

            { add to the listbox }
            newcount := PageList.Items.AddObject(TempInfo^.Display, Pointer(TempInfo));
            { check the format }
            PageFormatCheck(newcount);
         End
         Else If (pgcnt >= 255) Then
            Break;

         { next page to look for }
         Inc(pgcnt);

      Until (1 <> 1);
      { set the number of pages in device }
      MemPages := pgcnt;
   Finally
      { free up the string list memory }
      TempStrs.Free;
   End;

   { success if at least one page loaded }
   LoadiButtonFileMap := (PageList.Items.Count > 0);
End;


{----------------------------------------------------------------------------}
{ Take a normal string and parce it into the a byte buffer.  Return the
  Length or -1 for an error.  The Buf must be at least 128 bytes long.
}
Function TiBGenForm.ConvertToBin(instr: String; Buf : PChar): SmallInt;
Var
   cnt,ln,i,rcnt,addchar : SmallInt;
   By : Byte;
   lastchar : Char;
Begin
   ln := Length(instr);
   rcnt := 0;  { string run count }
   cnt := 0;   { position count in the buffer }
   addchar := 0;
   lastchar := Char(0);

   { loop to convert the chars in the string }
   For i := 1 to ln Do
   Begin
      By := HexCharToByte(instr[i]);

      { check if not to a space yet }
      If (instr[i] <> ' ') Then
      Begin
         { check if this is a non-hex }
         If (By = $FF) Then
         Begin
            If (rcnt > 0) Then
            Begin
               cnt := -1;
               Break;
            End;
         End
         Else
         Begin
            addchar := addchar shl 4;
            addchar := addchar Or By;
            If (rcnt > 1) Then
            Begin
               cnt := -1;
               Break;
            End;
         End;
         { count this char }
         lastchar := instr[i];
         Inc(rcnt);
      End
      { add it to the buffer }
      Else If (rcnt > 0) Then
      Begin
         If (rcnt = 2) Then
            Buf[cnt] := Chr(addchar)
         Else
            Buf[cnt] := lastchar;
         Inc(cnt);
         rcnt := 0;
         addchar := 0;
      End;
   End;

   { check for final run without trailing space }
   If ((cnt <> -1) And (rcnt > 0)) Then
   Begin
      If (rcnt = 2) Then
         Buf[cnt] := Chr(addchar)
      Else
         Buf[cnt] := lastchar;
      Inc(cnt);
   End;

   { return the length or -1 }
   ConvertToBin := cnt;
End;

{----------------------------------------------------------------------------}
{ Take a hex character and return the binary represetation
}
Function TiBGenForm.HexCharToByte(ch : Char): Byte;
Begin
   If ((Ord(ch) >= Ord('0')) And (Ord(ch) <= Ord('9'))) Then
      HexCharToByte := Byte(Ord(ch) - $30)
   Else If ((Ord(ch) >= Ord('A')) And (Ord(ch) <= Ord('F'))) Then
      HexCharToByte := Byte(Ord(ch) - $37)
   Else If ((Ord(ch) >= Ord('a')) And (Ord(ch) <= Ord('f'))) Then
      HexCharToByte := Byte(Ord(ch) - $57)
   Else
      HexCharToByte := $FF;
End;

{----------------------------------------------------------------------------}
{ Timer where work is done.
}
procedure TiBGenForm.IdleTimerTimer(Sender: TObject);
Var
   flag,i,m : SmallInt;
   tstr : String;
   PInfo : PPageInfo;
Begin
   { disable timer while sevicing }
   IdleTimer.Enabled := False;

   { check the 1-Wire port }
   If (DBMode = DB_CHECKPORT) Then
   Begin
      If (PortForm.PortIdle) Then
      Begin
         { set the port label  }
         StatusBar.SimpleText := PortForm.PortName;
         DBMode := DB_IDLE;
         { Success, so check for AutoMode, start write }
         If (AutoMode) Then
         Begin
            { set to destination page }
            NoteBook.ActivePage := DestinationPage;
            { set state to start writing }
            WriteButton.Caption := 'Cancel Write';
            DBMode := DB_FINDDEST;
            ROMDestLabel.Caption := '';
            TypeDestLabel.Caption := '';
            StatusLabel.Caption := '';
         End;
      End;
   End
   { find a source device }
   Else If (DBMode = DB_FINDSOURCE) Then
   Begin
      If (FindPart('') = TRUE) Then
      Begin
         { check for a device with memory }
         If (TypeList[ROM[0] And $7F].MemType = MEM_NONE) OR
            (TypeList[ROM[0] And $7F].MemType = MEM_SECURE) Then
         Begin
            m := DBMode;
            DBMode := DB_IDLE;
            MessageBeep(MB_OK);
            ShowMessage('ERROR, iButton type does not have memory or is unknown, choose another!');
            DBMode := m;
         End
         Else
         Begin
            MemPages := TypeList[ROM[0] And $7F].MemPages;
            MemType := TypeList[ROM[0] And $7F].MemType;
            DBMode := DB_READ;
         End;
      End;
   End
   { find a destination device }
   Else If (DBMode = DB_FINDDEST) Then
   Begin
      If (FindPart(RequiredSNLabel.Text) = TRUE) Then
      Begin
         { check for a device with memory }
         If (TypeList[ROM[0] And $7F].MemType <> MEM_NV) And
            (TypeList[ROM[0] And $7F].MemType <> MEM_EPROM) And
            (TypeList[ROM[0] And $7F].MemType <> MEM_EE) And
            (TypeList[ROM[0] And $7F].MemType <> MEM_MNV) Then
         Begin
            m := DBMode;
            DBMode := DB_IDLE;
            MessageBeep(MB_OK);
            ShowMessage('ERROR, iButton destination device not supported!');
            DBMode := m;
         End
         Else
         Begin
            MemPages := TypeList[ROM[0] And $7F].MemPages;
            MemType := TypeList[ROM[0] And $7F].MemType;

            { get a pointer to the last page object }
            PInfo := PPageInfo(PageList.Items.Objects[PageList.Items.Count - 1]);
            If (PInfo^.Pg >= MemPages) Then
            Begin
               m := DBMode;
               DBMode := DB_IDLE;
               MessageBeep(MB_OK);
               ShowMessage('ERROR, destination device does not have the necessary pages!');
               DBMode := m;
            End
            { ok we will try to program this device }
            Else
            Begin
               TypeDestLabel.Caption := 'iButton Type: ' + TypeList[ROM[0] And $7F].Name;
               ROMDestLabel.Caption := 'ROM: ';
               For i := 7 downto 0 do
                  ROMDestLabel.Caption := ROMDestLabel.Caption + IntToHex(ROM[i],2);
               { proceed }
               CurrentListPg := 0;
               If (TypeList[ROM[0] And $7F].MemType = MEM_EPROM) Then
                  DBMode := DB_EMPTY       { verify EPROM is empty on the pages }
               Else
               Begin
                  DBMode := DB_WRITENV;    { go to write of NVRAM pages }
                  { clear the write flags in the page list }
                  For i := 0 to PageList.Items.Count - 1 Do
                  Begin
                     { get a pointer to the page object }
                     PInfo := PPageInfo(PageList.Items.Objects[i]);
                     PInfo^.WriteCnt := 0;
                  End;
               End;
            End;
         End;
      End;
   End
   { Verify that the eprom pages are empty }
   Else If (DBMode = DB_EMPTY) Then
   Begin
      { get a pointer to the page object }
      PInfo := PPageInfo(PageList.Items.Objects[CurrentListPg]);

      { attempt to write the page }
      flag := Empty(PInfo^.Pg);

      { success so see fi more pages to write }
      If (flag = DBSUCCESS) Then
      Begin
         { mark the counter at the begining }
         PInfo^.WriteCnt := 0;
         { look for next page }
         Inc(CurrentListPg);
         { done? }
         If (CurrentListPg >= PageList.Items.Count) Then
         Begin
            DBMode := DB_WRITEEPROM;
            CurrentListPg := 0;
         End;
      End
      { page redirected }
      Else If (flag = DB_REDIRECT) Then
      Begin
         DBMode := DB_IDLE;
         MessageBeep(mb_IconExclamation);
         Application.MessageBox('ERROR, destination device''s required pages are redirected!'
                     ,AppName, mb_OK +
                    mb_IconExclamation + mb_ApplModal);
         DBMode := DB_FINDDEST;
      End
      { page already written }
      Else If (flag = DBNON_EMPTY) Then
      Begin
         DBMode := DB_IDLE;
         MessageBeep(mb_IconExclamation);
         Application.MessageBox('ERROR, destination device''s required pages are not empty!'
                     ,AppName, mb_OK +
                    mb_IconExclamation + mb_ApplModal);
         DBMode := DB_FINDDEST;
      End
      { no device }
      Else if (flag = DBNO_DEVICE) Then
         NoDevice(True)
      { other error }
      Else If ((flag < 0) And (flag <> DBNO_SESSION)) Then
      Begin
         { count consistent errors }
         If (ErrorType <> flag) Then
         Begin
            ErrorCnt := 0;
            ErrorType := flag;
         End
         Else
            Inc(ErrorCnt);

         { if long string of errors, then report }
         If (ErrorCnt > 10) Then
         Begin
            ErrorCnt := 0;
            DBMode := DB_IDLE;
            MessageBeep(mb_IconExclamation);
            Application.MessageBox('Could not read the device!'
                        ,AppName, mb_OK +
                       mb_IconExclamation + mb_ApplModal);
            DBMode := DB_FINDDEST;
         End;
      End;
      { reset the no device if needed }
      If (flag <> DBNO_DEVICE) Then
         NoDevice(False);
   End
   { Write eprom pages }
   Else If (DBMode = DB_WRITEEPROM) Then
   Begin
      { program the contents of the list page }
      flag := ProgramPage(CurrentListPg);

      { success so see fi more pages to write }
      If (flag = DBSUCCESS) Then
      Begin
         { look for next page }
         Inc(CurrentListPg);
         { done? }
         If (CurrentListPg >= PageList.Items.Count) Then
         Begin
            MessageBeep(MB_OK);
            WriteButton.Caption := 'Write Destination &iButton';
            StatusLabel.Caption := '';
            DBMode := DB_IDLE;
            { if in auto mode then close application when done }
            If (AutoMode) Then
               Application.Terminate
            Else
               Application.MessageBox('Copy Complete!'
                     ,AppName, mb_OK + + MB_ICONINFORMATION	+ mb_ApplModal);
         End;
      End
      { no device }
      Else if (flag = DBNO_DEVICE) Then
         NoDevice(True)
      { other error }
      Else If ((flag < 0) And (flag <> DBNO_SESSION)) Then
      Begin
         { count consistent errors }
         If (ErrorType <> flag) Then
         Begin
            ErrorCnt := 0;
            ErrorType := flag;
         End
         Else
            Inc(ErrorCnt);

         { if long string of errors, then report }
         If (ErrorCnt > 10) Then
         Begin
            ErrorCnt := 0;
            DBMode := DB_IDLE;
            MessageBeep(mb_IconExclamation);
            Application.MessageBox('Could not program the ERPOM device, do you have the correct hardware?'
                        ,AppName, mb_OK +
                       mb_IconExclamation + mb_ApplModal);
            DBMode := DB_FINDDEST;
         End;
      End;
      { reset the no device if needed }
      If (flag <> DBNO_DEVICE) Then
         NoDevice(False);
   End
   { Write the NVRAM device }
   Else If (DBMode = DB_WRITENV) Then
   Begin
      { get a pointer to the page object }
      PInfo := PPageInfo(PageList.Items.Objects[CurrentListPg]);

      { attempt to write the page }
      If (PInfo^.PgType = APPL) Then
      Begin
         StatusLabel.Caption := 'Writing Application Register ';
         flag := WriteApplicationReg(@PInfo^.PgData[0]);
         If (flag = DBSUCCESS) Then
            flag := LockApplicationReg;
      End
      Else
      Begin
         StatusLabel.Caption := 'Writing page ' + IntToStr(PInfo^.Pg);
         flag := WritePage(PInfo^.Pg, @PInfo^.PgData[0], PInfo^.PgLen);
      End;

      { success so see if more pages to write }
      If (flag = DBSUCCESS) Then
      Begin
         { mark this as done }
         PInfo^.WriteCnt := 32;
         { look for next page }
         Inc(CurrentListPg);
         { done? }
         If (CurrentListPg >= PageList.Items.Count) Then
         Begin
            MessageBeep(MB_OK);
            WriteButton.Caption := 'Write Destination &iButton';
            StatusLabel.Caption := '';
            DBMode := DB_IDLE;
            { if in auto mode then close application when done }
            If (AutoMode) Then
               Application.Terminate
            Else
               Application.MessageBox('Copy Complete!'
                     ,AppName, mb_OK + + MB_ICONINFORMATION	+ mb_ApplModal);
         End;
      End
      { no device }
      Else if (flag = DBNO_DEVICE) Then
         NoDevice(True)
      { other error }
      Else If ((flag < 0) And (flag <> DBNO_SESSION)) Then
      Begin
         { count consistent errors }
         If (ErrorType <> flag) Then
         Begin
            ErrorCnt := 0;
            ErrorType := flag;
         End
         Else
            Inc(ErrorCnt);

         { if long string of errors, then report }
         If (ErrorCnt > 10) Then
         Begin
            ErrorCnt := 0;
            DBMode := DB_IDLE;
            MessageBeep(mb_IconExclamation);
            Application.MessageBox('Could not write the device!'
                        ,AppName, mb_OK +
                       mb_IconExclamation + mb_ApplModal);
         End;
      End;
      { reset the no device if needed }
      If (flag <> DBNO_DEVICE) Then
         NoDevice(False);
   End
   { read the MEMORY }
   Else If (DBMode = DB_READ) Then
   Begin
      flag := ReadMemory;

      { done so go into slow verify mode }
      If (flag = DBDONE) Then
      Begin
         { recalculate CRC to see if I have BIG packets }
         ReCalPackets;

         { reprint current packet }
         PageList.ItemIndex := 0;
         DisplayPageItem(PageList.ItemIndex);

         { reset the button }
         ReadButton.Caption := 'Read Source &iButton';

         TypeLabel.Caption := 'iButton Type: ' + TypeList[ROM[0] And $7F].Name;
         ROMLabel.Caption := 'ROM: ';
         For i := 7 downto 0 do
            ROMLabel.Caption := ROMLabel.Caption + IntToHex(ROM[i],2);
         DBMode := DB_IDLE;

         MessageBeep(MB_OK);
      End
      { no device }
      Else if (flag = DBNO_DEVICE) Then
         NoDevice(True)
      { other error }
      Else If ((flag < 0) And (flag <> DBNO_SESSION)) Then
      Begin
         { count consistent errors }
         If (ErrorType <> flag) Then
         Begin
            ErrorCnt := 0;
            ErrorType := flag;
         End
         Else
            Inc(ErrorCnt);

         { if long string of errors, then report }
         If (ErrorCnt > 10) Then
         Begin
            ErrorCnt := 0;
            DBMode := DB_VERIFY;
            MessageBeep(mb_IconExclamation);
            Application.MessageBox('Could not read the device!'
                        ,AppName, mb_OK +
                       mb_IconExclamation + mb_ApplModal);
         End;
      End;
      { reset the no device if needed }
      If (flag <> DBNO_DEVICE) Then
         NoDevice(False);
   End;

   { re-enable timer }
   IdleTimer.Enabled := True;

end;

{----------------------------------------------------------------------------}
{ Write a RAW application register
}
Function TiBGenForm.WriteApplicationReg(ApplBuf : PChar): SmallInt;
Var
   rt,flag : SmallInt;
   BufCnt,StartData,Cnt,i : SmallInt;
   Buf : array[0..50] of Byte;      { array of page data }
   TargetBuf : array[0..50] of Byte;      { array of page data }
   TargetLen : SmallInt;
Begin
   rt := DBNO_DEVICE;

   {attempt to get a session }
   SHandle := TMExtendedStartSession(PortNum,PortType,nil);
   If (SHandle > 0) Then
   Begin
      { access and verify it is there }
      flag := TMStrongAccess(SHandle,@StateBuf);
      If (flag = 1) Then
      Begin
         { build a packet }
         BufCnt := 0;
         { write scratch command }
         Buf[BufCnt] := $99;
         Inc(BufCnt);
         { address 1 }
         Buf[BufCnt] := $00;
         Inc(BufCnt);
         { loop to write the data }
         For i := 0 to 7 Do
         Begin
            Buf[BufCnt] := Byte(ApplBuf[i]);
            Inc(BufCnt);
            { keep a copy of the target }
            TargetBuf[i] := Byte(ApplBuf[i]);
         End;

         { send/rec the packet }
         flag := TMBlockStream(SHandle, @Buf[0], BufCnt);
         { check results of block }
         If (flag = BufCnt) Then
           rt := DBSUCCESS;
      End
      Else
         rt := DBNO_DEVICE;
   End
   Else
     rt := DBNO_SESSION;

   { if so far success then attempt to read back }
   If (rt = DBSUCCESS) Then
   Begin
      rt := DBBAD_WRITE;
      For Cnt := 0 to 5 Do
      Begin
         { read back }
         flag := TMStrongAccess(SHandle,@StateBuf);
         If (flag = 1) Then
         Begin
            { build a packet }
            BufCnt := 0;
            { READ scratch command }
            Buf[BufCnt] := $C3;
            Inc(BufCnt);
            { address 1 }
            Buf[BufCnt] := $00;
            Inc(BufCnt);
            { loop to write the data }
            For i := 0 to 7 Do
            Begin
               Buf[BufCnt] := $FF;
               Inc(BufCnt);
            End;

            { send/rec the packet }
            flag := TMBlockStream(SHandle, @Buf[0], BufCnt);
            { check results of block }
            If (flag <> BufCnt) Then
              rt := DBNO_DEVICE
            Else
            Begin
               { compare }
               rt := DBSUCCESS;
               For i := 0 to 7 Do
               Begin
                  If (TargetBuf[i] <> Buf[i+2]) Then
                     rt := DBBAD_WRITE;
               End;
               If (rt = DBSUCCESS) Then
                  Break;
           End;
         End
         Else
            rt := DBNO_DEVICE;
      End;
   End;

   { close the opened session }
   TMEndSession(SHandle);

   WriteApplicationReg := rt;
End;

{----------------------------------------------------------------------------}
{ Lock the application register
}
Function TiBGenForm.LockApplicationReg: SmallInt;
Var
   rt,flag : SmallInt;
   BufCnt,StartData,Cnt,i : SmallInt;
   Buf : array[0..50] of Byte;      { array of page data }
   M : LongInt;
Begin
   rt := DBSUCCESS;

   {attempt to get a session }
   SHandle := TMExtendedStartSession(PortNum,PortType,nil);
   If (SHandle > 0) Then
   Begin
      { access and verify it is there }
      flag := TMStrongAccess(SHandle,@StateBuf);
      If (flag = 1) Then
      Begin
         { build a packet }
         BufCnt := 0;
         { copy and lock command }
         Buf[BufCnt] := $5A;
         Inc(BufCnt);
         { address 1 }
         Buf[BufCnt] := $A5;
         Inc(BufCnt);

         { send/rec the packet }
         flag := TMBlockStream(SHandle, @Buf[0], BufCnt);
         { check results of block }
         If (flag <> BufCnt) Then
           rt := DBNO_DEVICE;

         { wait for a while 100ms }
         M := GetTickCount + 100;
         While (M > GetTickCount) Do;
      End
      Else
         rt := DBNO_DEVICE;

      { close the opened session }
      TMEndSession(SHandle);
   End
   Else
     rt := DBNO_SESSION;

   LockApplicationReg := rt;
End;

{----------------------------------------------------------------------------}
{ Lock the application register
}
procedure TiBGenForm.NoDevice(state : Boolean);
Begin
   { set or clear the no-device symbol }
   NoDeviceLabel.Visible := state;
End;

procedure TiBGenForm.NoteBookChanging(Sender: TObject;
  var AllowChange: Boolean);
begin
end;

{----------------------------------------------------------------------------}
{ Notebook tab change
}
procedure TiBGenForm.NoteBookChange(Sender: TObject);
begin
   { Cancel Read or write if switch page }
   If (NoteBook.ActivePage <> SourcePage) And
      (ReadButton.Caption = 'Cancel Read') Then
      ReadButtonClick(Sender);
   If (NoteBook.ActivePage <> DestinationPage) And
      (WriteButton.Caption = 'Cancel Write') Then
      WriteButtonClick(Sender);
end;

end.
