{--------------------------------------------------------------
  Registry editing
}
unit regun;

interface

Uses
   WinProcs,SysUtils, GenGlob;

Function ReadRegStr(ValueName,DefaultValue: String): String;
Function ReadRegLong(ValueName: String; DefaultValue: LongInt): LongInt;
Function ReadTMEXRegStr(ValueName,DefaultValue: String): String;
Function WriteRegLong(ValueName: String; LongValue: LongInt): Boolean;
Function WriteRegStr(ValueName,StringValue: String): Boolean;

implementation


{----------------------------------------------------------------------------
  Read a string from the registry
}
Function ReadRegStr(ValueName,DefaultValue: String): String;
Var
   GlobKey : HKEY;
   rtval : String;
   RtType,RtSize : DWORD;
   KeyBuf : Array[0..255] of Char;
   RetBuf : Array[0..255] of Char;
Begin
   { set the value name }
   StrPCopy(KeyBuf,ValueName);
   { default value }
   rtval := DefaultValue;
   { get a hkey to the global area }
   If (RegOpenKeyEx(HKEY_CURRENT_USER,
          'Software\Dallas Semiconductor\iButton Copy Machine\3.10',
           0,KEY_READ,GlobKey) = ERROR_SUCCESS) Then
   Begin
      { read the value under the key }
      RtSize := 255;
      If (RegQueryValueEx(GlobKey,KeyBuf,nil,@RtType,
             @RetBuf[0],@RtSize) = ERROR_SUCCESS) Then
      Begin
         { check to verify that the return type is string }
         If (RtType = REG_SZ) Then
            rtval := StrPas(RetBuf); { success! }
      End;
      { release the key }
      RegCloseKey(GlobKey);
   End;
   { set return value }
   ReadRegStr := rtval;
End;

{----------------------------------------------------------------------------
  Read a string from the registry
}
Function ReadTMEXRegStr(ValueName,DefaultValue: String): String;
Var
   GlobKey : HKEY;
   rtval : String;
   RtType,RtSize : DWORD;
   KeyBuf : Array[0..255] of Char;
   RetBuf : Array[0..255] of Char;
Begin
   { set the value name }
   StrPCopy(KeyBuf,ValueName);
   { default value }
   rtval := DefaultValue;
   { get a hkey to the global area }
   If (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
          'Software\Dallas Semiconductor\iButton TMEX\3.00',
           0,KEY_READ,GlobKey) = ERROR_SUCCESS) Then
   Begin
      { read the value under the key }
      RtSize := 255;
      If (RegQueryValueEx(GlobKey,KeyBuf,nil,@RtType,
             @RetBuf[0],@RtSize) = ERROR_SUCCESS) Then
      Begin
         { check to verify that the return type is string }
         If (RtType = REG_SZ) Then
            rtval := StrPas(RetBuf); { success! }
      End;
      { release the key }
      RegCloseKey(GlobKey);
   End;
   { set return value }
   ReadTMEXRegStr := rtval;
End;


{----------------------------------------------------------------------------
  Read a Long from the registry
}
Function ReadRegLong(ValueName: String; DefaultValue: LongInt): LongInt;
Var
   GlobKey : HKEY;
   rtval : LongInt;
   RtType,RtSize : DWORD;
   KeyBuf : Array[0..255] of Char;
   RetBuf : LongInt;
Begin
   { set the value name }
   StrPCopy(KeyBuf,ValueName);
   { default value }
   rtval := DefaultValue;
   { get a hkey to the global area }
   If (RegOpenKeyEx(HKEY_CURRENT_USER,
          'Software\Dallas Semiconductor\iButton Copy Machine\3.10',
           0,KEY_READ,GlobKey) = ERROR_SUCCESS) Then
   Begin
      { read the value under the key }
      RtSize := 4;
      If (RegQueryValueEx(GlobKey,KeyBuf,nil,@RtType,
             @RetBuf,@RtSize) = ERROR_SUCCESS) Then
      Begin
         { check to verify that the return type is string }
         If (RtType = REG_DWORD) Then
            rtval := RetBuf; { success! }
      End;
      { release the key }
      RegCloseKey(GlobKey);
   End;
   { set return value }
   ReadRegLong := rtval;
End;

{----------------------------------------------------------------------------
  Write a Long from the registry
}
Function WriteRegLong(ValueName: String; LongValue: LongInt): Boolean;
Var
   SoftKey,DDKey,EgamiKey,OneKey : HKEY;
   rtval : Boolean;
   Exhist : DWORD;
   KeyBuf : Array[0..255] of Char;
Begin
   rtval := False;
   { set the value name }
   StrPCopy(KeyBuf,ValueName);
   { could not open global hkey }
   If (RegOpenKeyEx(HKEY_CURRENT_USER,'Software',0,
        KEY_WRITE,SoftKey) = ERROR_SUCCESS) Then
   Begin
      If (RegCreateKeyEx(SoftKey,'Dallas Semiconductor',0,'Per-User Data',
             REG_OPTION_NON_VOLATILE,KEY_WRITE,nil,DDKey,@Exhist)
             = ERROR_SUCCESS) Then
      Begin
         If (RegCreateKeyEx(DDKey,'iButton Copy Machine',0,'Per-User Data',
             REG_OPTION_NON_VOLATILE,KEY_WRITE,nil,EgamiKey,@Exhist)
             = ERROR_SUCCESS) Then
         Begin
            If (RegCreateKeyEx(EgamiKey,'3.10',0,'Per-User Data',
                REG_OPTION_NON_VOLATILE,KEY_WRITE,nil,OneKey,@Exhist)
                = ERROR_SUCCESS) Then
            Begin
               { set the value under the key }
               If (RegSetValueEx(OneKey,KeyBuf,0,REG_DWORD,@LongValue,
                    SizeOf(LongValue)) = ERROR_SUCCESS) Then
                    rtval := True;
               RegCloseKey(OneKey);
            End;
            RegCloseKey(EgamiKey);
         End;
         RegCloseKey(DDKey);
      End;
      RegCloseKey(SoftKey);
   End;
   { set the return value }
   WriteRegLong := rtval;
End;

{----------------------------------------------------------------------------
  Write a String from the registry
}
Function WriteRegStr(ValueName,StringValue: String): Boolean;
Var
   SoftKey,DDKey,EgamiKey,OneKey : HKEY;
   rtval : Boolean;
   Exhist : DWORD;
   KeyBuf : Array[0..255] of Char;
   WrBuf : Array[0..255] of Char;
Begin
   rtval := False;
   { set the value name }
   StrPCopy(KeyBuf,ValueName);
   StrPCopy(WrBuf,StringValue);
   { could not open global hkey }
   If (RegOpenKeyEx(HKEY_CURRENT_USER,'Software',0,
        KEY_WRITE,SoftKey) = ERROR_SUCCESS) Then
   Begin
      If (RegCreateKeyEx(SoftKey,'Dallas Semiconductor',0,'Per-User Data',
             REG_OPTION_NON_VOLATILE,KEY_WRITE,nil,DDKey,@Exhist)
             = ERROR_SUCCESS) Then
      Begin
         If (RegCreateKeyEx(DDKey,'iButton Copy Machine',0,'Per-User Data',
             REG_OPTION_NON_VOLATILE,KEY_WRITE,nil,EgamiKey,@Exhist)
             = ERROR_SUCCESS) Then
         Begin
            If (RegCreateKeyEx(EgamiKey,'3.10',0,'Per-User Data',
                REG_OPTION_NON_VOLATILE,KEY_WRITE,nil,OneKey,@Exhist)
                = ERROR_SUCCESS) Then
            Begin
               { set the value under the key }
               If (RegSetValueEx(OneKey,KeyBuf,0,REG_SZ,@WrBuf[0],
                    Length(StringValue)) = ERROR_SUCCESS) Then
                    rtval := True;
               RegCloseKey(OneKey);
            End;
            RegCloseKey(EgamiKey);
         End;
         RegCloseKey(DDKey);
      End;
      RegCloseKey(SoftKey);
   End;
   { set the return value }
   WriteRegStr := rtval;
End;


end.
